@@ -14,25 +14,26 @@ import os, shutil, string, sys, time
 class Mooi:
 	DIR = os.path.expandvars('${HOME}/Lavengro')
 	LRC = os.path.expandvars(DIR + os.sep + 'lavengrorc')
-	HDR = "\nLavengro - The Word Master\n" +\
-		  "-- Enter 'q' at any time to quit --\n"
-	SUB =  "(Available subjects) - 'q' to quit"
+	HDR = "\n\nLavengro - The Word Master\n\n" 
+	SUB =  "(Available subjects)"
 	MODE = ['lavengro - programmed learning',
 			'test - one chance to answer, scored result',
 			'tutor - retest until all correct, no score',
-			'quiz - bsd compatible, uses index file in program dir']
+			'quiz - bsd compatible, uses index file (see docs)']
 
 	def __init__(self, debug):
-		self.debug = debug
+		self.header = 0
 		self.lav = None
 		self.lavLoaded = 0
 		self.modes = [0,0,0,0]				# lavengro, test, tutor, quiz
-		self.prevUsrChoice = -1
 		self.oldLav = []
 		self.pending = []
+		self.prevUsrChoice = -1
 		self.quiz = []
 		self.record = ''
 		self.scored = []
+		self.subdir = None
+		self.subject = None
 		self.taken = []
 		self.tests = None
 		self.total = 0
@@ -59,20 +60,21 @@ class Mooi:
 
 	# get user to choose an element from thing
 	def query(self, thing, prompt):
+		print "\n\n--- 'q' to quit ---"
 		length = len(thing)
-		choice = length
+		choice = length + 1
 		for i in range(0, length):
 			tokens = string.split(thing[i],':')
-			print '(' + `i` + ') ' + string.strip(tokens[0])
-		while ((choice >= length) or (choice == self.prevUsrChoice)):
+			print '(' + `i+1` + ') ' + string.strip(tokens[0])
+		while ((choice > length) or (choice == self.prevUsrChoice)):
 			choice = raw_input(prompt)
 			if (choice == 'q'):
 				self.exit(' ', 0)
 			else:
 				try:
-					choice = int(choice)
+					choice = int(choice) - 1
 				except:
-					choice = 666
+					choice = length + 1
 		return choice
 
 	#compare files by date, oldest first
@@ -126,35 +128,36 @@ class Mooi:
 
 	# get quiz in lavengro mode, choosing dir then file
 	def selectQuiz(self):
-		self.prevUsrChoice = -1	
-
-		if (self.modes[3]):
-			self.bsdMode()
-			return			
-
-		if ((self.modes[0]) and (self.tests != None)): # subsequent lavengro loads
+		# subsequent lavengro loads
+		if ((self.modes[0]) and (self.tests != None)): 
 			if (len(self.tests) != 0):
 				self.quiz = [ self.prefix + self.get_next(), 0, 1]
 				return
 			else:
 				self.exit('All tests taken.', 0)
-			
 		# choose dir/subject
-		dirs = []
-		files = os.listdir(self.DIR)
-		for file in files:
-			if ((os.path.isdir(self.DIR + os.sep + file)) and
-				(file != 'py')):
-				dirs.append(file)
-		print self.SUB
-		choice = self.query(dirs, 'Choose subject: ')
-		subdir = dirs[choice]
-
+		if (self.subject == None):
+			dirs = []
+			files = os.listdir(self.DIR)
+			for file in files:
+				if ((os.path.isdir(self.DIR + os.sep + file)) and
+					(file != 'py')):
+					dirs.append(file)
+			print self.SUB
+			self.subject = choice = self.query(dirs, 'Choose subject: ')
+			self.subdir = dirs[choice]
+		# set testing mode
+		self.modes = [0,0,0,0]
+		self.modes[self.query(self.MODE, 'Select testing mode: ')] = 1
+		# bsd is its own thing
+		if (self.modes[3]):
+			self.bsdMode()
+			return			
 		# choose test in subject
 		self.tests = []
-		files = os.listdir(self.DIR + os.sep + subdir)
+		files = os.listdir(self.DIR + os.sep + self.subdir)
 		for file in files:
-			if (not os.path.isdir(self.DIR + os.sep + subdir + os.sep + file)):
+			if (not os.path.isdir(self.DIR + os.sep + self.subdir + os.sep + file)):
 				# no backups, *MEs, .lavs, or bsd indices
 				if ((string.find(file,'~') == -1) and
 					(string.find(file,'.lav') == -1) and
@@ -163,10 +166,8 @@ class Mooi:
 					self.tests.append(file)
  		if (len(self.tests) == 0):
 			self.exit('No tests for chosen subject.\nExiting...', 1)
-
-		self.prefix = self.DIR + os.sep + subdir + os.sep
+		self.prefix = self.DIR + os.sep + self.subdir + os.sep
 		self.tests.sort(self.compDates)
-
 		user = os.path.expandvars('$USER')
 		if (user == ''):
 			user = 'user'
@@ -178,10 +179,8 @@ class Mooi:
 			else:					   
 				self.lav = open(self.record, 'w')
 			self.quiz = [ self.prefix + self.get_next(), 0, 1]
-		else:
-			if not self.lavLoaded:
-				self.loadLav()
-				self.lavLoaded = 1
+		else:							# all test and tutor loads
+			if not self.lavLoaded: self.loadLav()
 			self.tests.sort()
 			choice = self.query(self.tests, 'Enter quiz number: ')
 			self.quiz = [ self.prefix + self.tests[choice], 0, 1]
@@ -208,10 +207,12 @@ class Mooi:
 			self.oldLav = []
 			lines = []
 		self.lav = open(self.record, 'w')
+		self.lavLoaded = 1
 		return lines
 
 	# get quiz in bsd mode, choosing file, questions, and answers
 	def bsdMode(self):
+		self.prevUsrChoice = -1	
 		self.quiz = []	
 		# get bsd index
 		f = open(self.DIR + os.sep + 'index')
@@ -286,10 +287,9 @@ class Mooi:
 	# - Lavengro - #
 	# load data for Lavengro instance
 	def loadData(self, data):
-		if not self.modes[0]:
+		if not (self.header):
 			print self.HDR
-			self.modes = [0,0,0,0]
-			self.modes[self.query(self.MODE, 'Select testing mode: ')] = 1
+			self.header = 1
 		self.selectQuiz()
 		self.loadQuiz(data)
 		return 
